@@ -28,7 +28,9 @@ def _block(sig):
         if attrs_:
             yield '(%s)' % ''.join(attrs_)
 
-        yield '. '
+        # We can safely always output extended blocks, because we always
+        # use the p. signature for unamed blocks. 
+        yield '.. '
 
         for i in el.contents:
             for j in walk_tree(i):
@@ -77,6 +79,11 @@ def _anchor(el):
         yield '"'
     if 'href' in attrs: yield ':%s ' % attrs['href']
     
+
+def _html(el):
+    """Return unrecognized tags as is."""
+    yield '%s\n\n' % el
+
     
 def walk_tree(el):
     tags = {'p'         : _block('p'),
@@ -101,12 +108,18 @@ def walk_tree(el):
 
     if getattr(el, 'name', None):
         # Tag.
-        try:
-            f = tags[el.name]
-            for i in f(el): yield i
-        except KeyError:
-            # Return unrecognized tags as is.
-            yield '%s\n\n' % el
+        f = tags.get(el.name, _html)
+        for i in f(el): yield i
+            
+        # Blocks must always be followed by another block, so if we
+        # add the default block signature if it's not the case.
+        if f.__name__ == 'block':
+            next = el.nextSibling
+            # Look for the first named sibling.
+            while not getattr(next, 'name', None) and next:
+                next = next.nextSibling
+                f = tags.get(next.name, None)
+                if f and f.__name__ != 'block': yield 'p. '
     else:
         # String.
         yield el
@@ -128,7 +141,7 @@ if __name__ == '__main__':
     input = """<html>
     <head><title>Page title</title></head>
     <body>
-    <blockquote>Test</blockquote>
+    <blockquote><div>Test</div></blockquote>
     <p id="firstpara" align="center" class="class1 class2">This is paragraph <b>one</b>.
     <p id="secondpara" align="blah">This is paragraph <b>two</b>.
 
@@ -136,6 +149,8 @@ if __name__ == '__main__':
 
     <pre>
     Some code
+
+    Some more code
     </pre>
 
     <img src="http://example.com/image.gif" alt="alternative description" />
